@@ -40,6 +40,7 @@
 #include <OpenIPMI/ipmi_types.h>
 #include <OpenIPMI/ipmi_bits.h>
 #include <OpenIPMI/ipmi_log.h>
+#include <OpenIPMI/ipmiif.h> /* for ipmi_args_t, sigh */
 
 #include <OpenIPMI/internal/ipmi_malloc.h>
 #include <OpenIPMI/internal/ipmi_locks.h>
@@ -112,17 +113,19 @@ void ipmi_set_uint16(unsigned char *data, int val);
    actual length.  The max_out_len is the maximum number of characters
    to output, including the nil.  The type will be set to either
    unicode or ASCII.  The number of bytes put into the output string
-   is returned.  The input pointer will be updated to point to the
-   character after the used data. */
+   is returned in out_len.  The input pointer will be updated to point
+   to the character after the used data.  This returns a standard
+   error value. */
 #define IPMI_STR_SDR_SEMANTICS	0
 #define IPMI_STR_FRU_SEMANTICS	1
-unsigned int ipmi_get_device_string(unsigned char        ** const input,
-				    unsigned int         in_len,
-				    char                 *output,
-				    int                  semantics,
-				    int                  force_unicode,
-				    enum ipmi_str_type_e *type,
-				    unsigned int         max_out_len);
+int ipmi_get_device_string(unsigned char        **input,
+			   unsigned int         in_len,
+			   char                 *output,
+			   int                  semantics,
+			   int                  force_unicode,
+			   enum ipmi_str_type_e *type,
+			   unsigned int         max_out_len,
+			   unsigned int         *out_len);
 
 /* Store an IPMI device string in the most compact form possible.
    input is the input string (nil terminated), output is where to
@@ -134,7 +137,7 @@ void ipmi_set_device_string(const char           *input,
 			    unsigned int         in_len,
 			    unsigned char        *output,
 			    int                  force_unicode,
-			    int                  *out_len);
+			    unsigned int         *out_len);
 
 /* Generate a log.  Note that logs should not end in a newline, that
    will be automatically added as needed to the log.  */
@@ -144,6 +147,54 @@ void ipmi_log(enum ipmi_log_type_e log_type, const char *format, ...)
 #endif
 ;
 
+/* Information for connection handlers. */
+typedef struct ipmi_con_setup_s ipmi_con_setup_t;
+typedef int (*ipmi_con_parse_args_cb)(int         *curr_arg,
+				      int         arg_count,
+				      char        * const *args,
+				      ipmi_args_t **iargs);
+typedef const char *(*ipmi_con_get_help_cb)(void);
+typedef ipmi_args_t *(*ipmi_con_alloc_args_cb)(void);
+ipmi_con_setup_t *_ipmi_alloc_con_setup(ipmi_con_parse_args_cb parse,
+					ipmi_con_get_help_cb   help,
+					ipmi_con_alloc_args_cb alloc);
+void _ipmi_free_con_setup(ipmi_con_setup_t *v);
+
+int _ipmi_register_con_type(const char *name, ipmi_con_setup_t *setup);
+int _ipmi_unregister_con_type(const char *name, ipmi_con_setup_t *setup);
+
+typedef void (*ipmi_args_free_cb)(ipmi_args_t *args);
+typedef int (*ipmi_args_connect_cb)(ipmi_args_t  *args,
+				    os_handler_t *handler,
+				    void         *user_data,
+				    ipmi_con_t   **new_con);
+typedef int (*ipmi_args_get_val_cb)(ipmi_args_t  *args,
+				    unsigned int argnum,
+				    const char   **name,
+				    const char   **type,
+				    const char   **help,
+				    char         **value,
+				    const char   ***range);
+typedef int (*ipmi_args_set_val_cb)(ipmi_args_t  *args,
+				    unsigned int argnum,
+				    const char   *name,
+				    const char   *value);
+typedef ipmi_args_t *(*ipmi_args_copy_cb)(ipmi_args_t *args);
+typedef int (*ipmi_args_validate_cb)(ipmi_args_t *args, int *argnum);
+typedef void (*ipmi_args_free_val_cb)(ipmi_args_t *args, char *value);
+typedef const char *(*ipmi_args_get_type_cb)(ipmi_args_t *args);
+ipmi_args_t *_ipmi_args_alloc(ipmi_args_free_cb     free,
+			      ipmi_args_connect_cb  connect,
+			      ipmi_args_get_val_cb  get_val,
+			      ipmi_args_set_val_cb  set_val,
+			      ipmi_args_copy_cb     copy,
+			      ipmi_args_validate_cb validate,
+			      ipmi_args_free_val_cb free_val,
+			      ipmi_args_get_type_cb get_type,
+			      unsigned int          extra_data_len);
+void *_ipmi_args_get_extra_data(ipmi_args_t *args);
+
+
 /* Internal function to get the name of a domain. */
 const char *_ipmi_domain_name(const ipmi_domain_t *domain);
 const char *_ipmi_mc_name(const ipmi_mc_t *mc);
