@@ -57,10 +57,6 @@ void _ipmi_sensor_put(ipmi_sensor_t *sensor);
 /* Return the number of sensors in the data structure. */
 unsigned int ipmi_sensors_get_count(ipmi_sensor_info_t *sensors);
 
-/* Get the waitq for the sensors repository.  This is used to
-   serialize SDR fetches. */
-opq_t *_ipmi_sensors_get_waitq(ipmi_sensor_info_t *sensors);
-
 /*
  * These are for OEM code to create their own sensors.
  */
@@ -270,11 +266,11 @@ void ipmi_sensor_threshold_set_settable(ipmi_sensor_t      *sensor,
    hot-swap.  This should generally be a slot sensor (sensor type
    21h). */
 int ipmi_sensor_is_hot_swap_requester(ipmi_sensor_t *sensor,
-				      unsigned int  *offset,
-				      unsigned int  *val_when_requesting);
+				      int           *offset,
+				      int           *val_when_requesting);
 void ipmi_sensor_set_hot_swap_requester(ipmi_sensor_t *sensor,
-					unsigned int  offset,
-					unsigned int  val_when_requesting);
+					int           offset,
+					int           val_when_requesting);
 
 /* Allow OEM code to call the event handlers.  Note that if the event
    is handled by the handlers, then "*event" will be set to NULL and
@@ -493,10 +489,6 @@ void ipmi_sensor_set_base_unit_string(ipmi_sensor_t *sensor,
 void ipmi_sensor_set_modifier_unit_string(ipmi_sensor_t *sensor,
 					  const char    *str);
 
-/* Get the MC that the message is sent to for reading and controlling
-   the sensor. */
-ipmi_mc_t *ipmi_sensor_get_mc(ipmi_sensor_t *sensor);
-
 /* Get the sensor's domain. */
 ipmi_domain_t *ipmi_sensor_get_domain(ipmi_sensor_t *sensor);
 
