@@ -88,17 +88,52 @@ typedef void (*ipmi_ll_con_changed_cb)(ipmi_con_t   *ipmi,
 /* Used when fetching the IPMB address of the connection. The active
    parm tells if the interface is active or not, this callback is also
    used to inform the upper layer when the connection becomes active
-   or inactive. */
-typedef void (*ipmi_ll_ipmb_addr_cb)(ipmi_con_t   *ipmi,
-				     int          err,
-				     unsigned int ipmb_addr,
-				     int          active,
-				     unsigned int hacks,
-				     void         *cb_data);
+   or inactive.  Note that there can be one IPMB address per channel,
+   so this allows an array of IPMBs to be passed, one per channel.
+   Set the IPMB to 0 if unknown. */
+typedef void (*ipmi_ll_ipmb_addr_cb)(ipmi_con_t    *ipmi,
+				     int           err,
+				     const unsigned char ipmb_addr[],
+				     unsigned int  num_ipmb_addr,
+				     int           active,
+				     unsigned int  hacks,
+				     void          *cb_data);
 
 /* Used to handle knowing when the connection shutdown is complete. */
 typedef void (*ipmi_ll_con_closed_cb)(ipmi_con_t *ipmi, void *cb_data);
 
+/* Statistics interfaces. */
+typedef struct ipmi_ll_stat_info_s ipmi_ll_stat_info_t;
+typedef void (*ipmi_ll_con_add_stat_cb)(ipmi_ll_stat_info_t *info,
+					void                *stat,
+					int                 count);
+typedef int (*ipmi_ll_con_register_stat_cb)(ipmi_ll_stat_info_t *info,
+					    const char          *name,
+					    const char          *instance,
+					    void                **stat);
+typedef void (*ipmi_ll_con_unregister_stat_cb)(ipmi_ll_stat_info_t *info,
+					       void                *stat);
+ipmi_ll_stat_info_t *ipmi_ll_con_alloc_stat_info(void);
+void ipmi_ll_con_free_stat_info(ipmi_ll_stat_info_t *info);
+void ipmi_ll_con_stat_info_set_adder(ipmi_ll_stat_info_t     *info,
+				     ipmi_ll_con_add_stat_cb adder);
+void ipmi_ll_con_stat_info_set_register(ipmi_ll_stat_info_t          *info,
+					ipmi_ll_con_register_stat_cb reg);
+void ipmi_ll_con_stat_info_set_unregister(ipmi_ll_stat_info_t            *info,
+					  ipmi_ll_con_unregister_stat_cb ureg);
+void ipmi_ll_con_stat_call_adder(ipmi_ll_stat_info_t *info,
+				 void                *stat,
+				 int                 count);
+int ipmi_ll_con_stat_call_register(ipmi_ll_stat_info_t *info,
+				   const char          *name,
+				   const char          *instance,
+				   void                **stat);
+void ipmi_ll_con_stat_call_unregister(ipmi_ll_stat_info_t *info,
+				      void                *stat);
+void ipmi_ll_con_stat_set_user_data(ipmi_ll_stat_info_t *info,
+				    void                *data);
+void *ipmi_ll_con_stat_get_user_data(ipmi_ll_stat_info_t *info);
+
 /* Set this bit in the hacks if, even though the connection is to a
    device not at 0x20, the first part of a LAN command should always
    use 0x20. */
@@ -116,6 +151,31 @@ typedef void (*ipmi_ll_con_closed_cb)(ipmi_con_t *ipmi, void *cb_data);
    of the spec. */
 #define IPMI_CONN_HACK_RMCPP_INTEG_SIK		0x00000004
 
+/*
+ * Used to pass special options for sending messages.
+ */
+typedef struct ipmi_con_option_s
+{
+    int option;
+    union {
+	long ival;
+	void *pval;
+    };
+} ipmi_con_option_t;
+
+/* Used to mark the end of the option list.  Must always be the last
+   option. */
+#define IPMI_CON_OPTION_LIST_END		0
+
+/* Enable/disable authentication on the message (set by ival).
+   Default is enabled. */
+#define IPMI_CON_MSG_OPTION_AUTH		1
+
+/* Enable/disable confidentiality (encryption) on the message (set by
+   ival).  Default is enabled. */
+#define IPMI_CON_MSG_OPTION_CONF		2
+
+
 /* The data structure representing a connection.  The low-level handler
    fills this out then calls ipmi_init_con() with the connection. */
 struct ipmi_con_s
@@ -169,7 +229,8 @@ struct ipmi_con_s
        use this to change it.  The hacks are the same as the ones in
        the IPMB address handler. */
     void (*set_ipmb_addr)(ipmi_con_t    *ipmi,
-			  unsigned char ipmb,
+			  const unsigned char ipmb_addr[],
+			  unsigned int  num_ipmb_addr,
 			  int           active,
 			  unsigned int  hacks);
 
@@ -287,7 +348,7 @@ struct ipmi_con_s
     unsigned int  hacks;
 
     /* The IPMB address as reported by the lower layer. */
-    unsigned char ipmb_addr;
+    unsigned char ipmb_addr[MAX_IPMI_USED_CHANNELS];
 
     /* Handle an async event for the connection reported by something
        else. */
@@ -302,12 +363,43 @@ struct ipmi_con_s
        ipmi_con_attr_cleanup() when the connection is destroyed. */
     void *attr;
 
-    /* Statistics interfaces.  These may be NULL if the user doesn't
-       want statistics.  They pass in the user data field. */
+    /* Old statistics interfaces.  Do not use these, they don't work
+       any more. */
     int (*register_stat)(void *user_data, char *name,
 			 char *instance,  void **stat);
     void (*add_stat)(void *user_data, void *stat, int value);
     void (*finished_with_stat)(void *user_data, void *stat);
+
+    /* Return the arguments or the connection. */
+    ipmi_args_t *(*get_startup_args)(ipmi_con_t *con);
+
+    /* Increment the usecount of the connection; for each use, the
+       connection must be closed.  This may be NULL if the connection
+       type does not support being reused. */
+    void (*use_connection)(ipmi_con_t *con);
+
+    /* Like send_command, but with options.  options may be NULL if
+       none.  If options are passed in, they must be terminated with
+       the proper option.  This field may be NULL if the connection
+       does not support options. */
+    int (*send_command_option)(ipmi_con_t              *ipmi,
+			       const ipmi_addr_t       *addr,
+			       unsigned int            addr_len,
+			       const ipmi_msg_t        *msg,
+			       const ipmi_con_option_t *options,
+			       ipmi_ll_rsp_handler_t   rsp_handler,
+			       ipmi_msgi_t             *rspi);
+
+    /* Returns the number of ports on the connection (one more than
+       the max_port that can be reported by ipmi_ll_con_changed_cb().
+       If NULL, assume 1. */
+    unsigned int (*get_num_ports)(ipmi_con_t *ipmi);
+
+    /* New statistics interface. */
+    int (*register_stat_handler)(ipmi_con_t          *ipmi,
+				 ipmi_ll_stat_info_t *info);
+    int (*unregister_stat_handler)(ipmi_con_t          *ipmi,
+				   ipmi_ll_stat_info_t *info);
 };
 
 #define IPMI_CONN_NAME(c) (c->name ? c->name : "")
