@@ -202,7 +202,7 @@ struct lmc_data_s
 
     sdrs_t main_sdrs;
     sdr_t  *part_add_sdr;
-    int    part_add_next;
+    unsigned int part_add_next;
     int    in_update_mode;
 
     unsigned char event_receiver;
@@ -636,6 +636,7 @@ handle_delete_sel_entry(lmc_data_t    *mc,
     ipmi_set_uint16(rdata+1, entry->record_id);
     *rdata_len = 3;
 
+    mc->sel.count--;
     free(entry);
 }
 
@@ -685,8 +686,10 @@ handle_clear_sel(lmc_data_t    *mc,
     }
 
     rdata[1] = 1;
-    if (op == 0) {
+    if (op == 0xaa) {
 	entry = mc->sel.entries;
+	mc->sel.entries = NULL;
+	mc->sel.count = 0;
 	while (entry) {
 	    n_entry = entry->next;
 	    free(entry);
@@ -981,10 +984,10 @@ handle_get_sdr(lmc_data_t    *mc,
 	       unsigned char *rdata,
 	       unsigned int  *rdata_len)
 {
-    uint16_t record_id;
-    int      offset;
-    int      count;
-    sdr_t    *entry;
+    uint16_t     record_id;
+    unsigned int offset;
+    unsigned int count;
+    sdr_t        *entry;
 
     if (!(mc->device_support & IPMI_DEVID_SDR_REPOSITORY_DEV)) {
 	handle_invalid_cmd(mc, rdata, rdata_len);
@@ -1107,9 +1110,9 @@ handle_partial_add_sdr(lmc_data_t    *mc,
 		       unsigned char *rdata,
 		       unsigned int  *rdata_len)
 {
-    uint16_t record_id;
-    int      offset;
-    int      modal;
+    uint16_t     record_id;
+    unsigned int offset;
+    int          modal;
 
     if (!(mc->device_support & IPMI_DEVID_SDR_REPOSITORY_DEV)) {
 	handle_invalid_cmd(mc, rdata, rdata_len);
@@ -1465,8 +1468,8 @@ handle_read_fru_data(lmc_data_t    *mc,
 		     unsigned int  *rdata_len)
 {
     unsigned char devid;
-    int           offset;
-    int           count;
+    unsigned int  offset;
+    unsigned int  count;
 
     if (check_msg_length(msg, 4, rdata, rdata_len))
 	return;
@@ -1509,8 +1512,8 @@ handle_write_fru_data(lmc_data_t    *mc,
 		      unsigned int  *rdata_len)
 {
     unsigned char devid;
-    int           offset;
-    int           count;
+    unsigned int  offset;
+    unsigned int  count;
 
     if (check_msg_length(msg, 3, rdata, rdata_len))
 	return;
@@ -1892,10 +1895,10 @@ handle_get_device_sdr(lmc_data_t    *mc,
 		      unsigned char *rdata,
 		      unsigned int  *rdata_len)
 {
-    uint16_t record_id;
-    int      offset;
-    int      count;
-    sdr_t    *entry;
+    uint16_t     record_id;
+    unsigned int offset;
+    unsigned int count;
+    sdr_t        *entry;
 
     if (!(mc->has_device_sdrs)) {
 	handle_invalid_cmd(mc, rdata, rdata_len);
@@ -3100,7 +3103,7 @@ handle_picmg_cmd_set_fru_led_state(lmc_data_t    *mc,
 				   unsigned char *rdata,
 				   unsigned int  *rdata_len)
 {
-    int led;
+    unsigned int led;
 
     if (check_msg_length(msg, 3, rdata, rdata_len))
 	return;
@@ -3168,7 +3171,7 @@ handle_picmg_cmd_get_fru_led_state(lmc_data_t    *mc,
 				   unsigned char *rdata,
 				   unsigned int  *rdata_len)
 {
-    int led;
+    unsigned int led;
 
     if (check_msg_length(msg, 3, rdata, rdata_len))
 	return;
