@@ -52,12 +52,12 @@ struct ipmi_control_info_s
     ipmi_control_t           **controls_by_idx;
     /* Size of above control array.  This will be 0 if the LUN has no
        controls. */
-    int                      idx_size;
+    unsigned int             idx_size;
 
-    ipmi_lock_t *idx_lock;
+    ipmi_lock_t              *idx_lock;
 
     /* Total number of controls we have in this. */
-    unsigned int control_count;
+    unsigned int             control_count;
 
     opq_t *control_wait_q;
     int  wait_err;
@@ -109,9 +109,10 @@ struct ipmi_control_s
     locked_list_t *handler_list;
 
     /* For light types. */
+#define MAX_LIGHTS 10
     ipmi_control_light_t *lights;
-    unsigned int         colors;
-    int                  has_local_control;
+    unsigned int         colors[MAX_LIGHTS];
+    int                  has_local_control[MAX_LIGHTS];
 
     /* For display types. */
     unsigned int columns;
@@ -367,6 +368,13 @@ ipmi_control_find_id(ipmi_domain_id_t domain_id,
  *
  **********************************************************************/
 
+static int
+control_ok_to_use(ipmi_control_t *control)
+{
+    return (   !control->destroyed
+	    && !_ipmi_domain_in_shutdown(control->domain));
+}
+
 static void
 control_final_destroy(ipmi_control_t *control)
 {
@@ -398,11 +406,18 @@ control_final_destroy(ipmi_control_t *control)
 int
 ipmi_control_destroy(ipmi_control_t *control)
 {
-    ipmi_control_info_t *controls = _ipmi_mc_get_controls(control->mc);
+    ipmi_control_info_t *controls;
+    ipmi_mc_t           *mc = control->mc;
+
+    _ipmi_domain_mc_lock(control->domain);
+    _ipmi_mc_get(mc);
+    _ipmi_domain_mc_unlock(control->domain);
+    controls = _ipmi_mc_get_controls(control->mc);
 
     ipmi_lock(controls->idx_lock);
     if (controls->controls_by_idx[control->num] != control) {
 	ipmi_unlock(controls->idx_lock);
+	_ipmi_mc_put(control->mc);
 	return EINVAL;
     }
 
@@ -415,6 +430,7 @@ ipmi_control_destroy(ipmi_control_t *control)
 
     control->destroyed = 1;
     _ipmi_control_put(control);
+    _ipmi_mc_put(mc);
 
     return 0;
 }
@@ -449,7 +465,7 @@ ipmi_control_get_name(ipmi_control_t *control, char *name, int length)
 
     if (control->entity)
 	rv = ipmi_entity_get_name(control->entity, name, length);
-    if (length > control->id_len + 2)
+    if (length > (int) (control->id_len + 2))
 	length = control->id_len + 2; /* Leave space for the nil */
     rv += snprintf(name+rv, length, ".%s", control->id);
     return rv;
@@ -469,7 +485,7 @@ control_opq_ready2(ipmi_control_t *control, void *cb_data)
 	info->__handler(control, 0, info->__cb_data);
 }
 
-static void
+static int
 control_opq_ready(void *cb_data, int shutdown)
 {
     ipmi_control_op_info_t *info = cb_data;
@@ -482,13 +498,14 @@ control_opq_ready(void *cb_data, int shutdown)
 		 CONTROL_NAME(info->__control));
 	if (info->__handler)
 	    info->__handler(info->__control, ECANCELED, info->__cb_data);
-	return;
+	return OPQ_HANDLER_STARTED;
     }
 
     rv = ipmi_control_pointer_cb(info->__control_id, control_opq_ready2, info);
     if (rv)
 	if (info->__handler)
 	    info->__handler(info->__control, rv, info->__cb_data);
+    return OPQ_HANDLER_STARTED;
 }
 
 int
@@ -517,7 +534,7 @@ ipmi_control_opq_done(ipmi_control_t *control)
     if (!control)
 	return;
 
-     /* This gets called on ECANCELLED error cases, if the sensor is
+     /* This gets called on ECANCELED error cases, if the control is
 	already we need to clear out the opq. */
     if (control->destroyed) {
 	if (control->waitq) {
@@ -550,6 +567,7 @@ control_rsp_handler(ipmi_mc_t  *mc,
     ipmi_control_op_info_t *info = rsp_data;
     int                    rv;
     ipmi_control_t         *control = info->__control;
+    ipmi_entity_t          *entity = NULL;
 
     if (control->destroyed) {
 	ipmi_entity_t *entity = NULL;
@@ -579,8 +597,22 @@ control_rsp_handler(ipmi_mc_t  *mc,
 	ipmi_log(IPMI_LOG_ERR_INFO,
 		 "control.c(control_rsp_handler): "
 		 "MC was destroyed while a control operation was in progress");
+
+	_ipmi_domain_entity_lock(control->domain);
+	control->usecount++;
+	_ipmi_domain_entity_unlock(control->domain);
+
+	rv = _ipmi_entity_get(control->entity);
+	if (! rv)
+	    entity = control->entity;
+
 	if (info->__rsp_handler)
 	    info->__rsp_handler(control, ECANCELED, NULL, info->__cb_data);
+
+	_ipmi_control_put(control);
+	if (entity)
+	    _ipmi_entity_put(entity);
+
 	return;
     }
 
@@ -590,12 +622,26 @@ control_rsp_handler(ipmi_mc_t  *mc,
 				 control_rsp_handler2,
 				 info);
     if (rv) {
+	int nrv;
+
 	ipmi_log(IPMI_LOG_ERR_INFO,
 		 "%scontrol.c(control_rsp_handler): "
 		 "Could not convert control id to a pointer",
 		 MC_NAME(mc));
+	_ipmi_domain_entity_lock(control->domain);
+	control->usecount++;
+	_ipmi_domain_entity_unlock(control->domain);
+
+	nrv = _ipmi_entity_get(control->entity);
+	if (! nrv)
+	    entity = control->entity;
+
 	if (info->__rsp_handler)
 	    info->__rsp_handler(control, rv, NULL, info->__cb_data);
+
+	_ipmi_control_put(control);
+	if (entity)
+	    _ipmi_entity_put(entity);
     }
 }
 			 
@@ -657,8 +703,13 @@ control_addr_response_handler(ipmi_domain_t *domain, ipmi_msgi_t *rspi)
 		 "%scontrol.c(control_addr_response_handler): "
 		 "Could not convert control id to a pointer",
 		 DOMAIN_NAME(domain));
-	if (info->__rsp_handler)
+	if (info->__rsp_handler) {
+	    _ipmi_domain_entity_lock(control->domain);
+	    control->usecount++;
+	    _ipmi_domain_entity_unlock(control->domain);
 	    info->__rsp_handler(control, rv, NULL, info->__cb_data);
+	    _ipmi_control_put(control);
+	}
     }
     return IPMI_MSG_ITEM_NOT_USED;
 }
@@ -773,7 +824,7 @@ ipmi_control_add_nonstandard(ipmi_mc_t               *mc,
     if (num >= controls->idx_size) {
 	ipmi_control_t **new_array;
 	unsigned int   new_size;
-	int            i;
+	unsigned int   i;
 
 	/* Allocate the array in multiples of 16 (to avoid thrashing malloc
 	   too much). */
@@ -851,7 +902,7 @@ ipmi_control_add_nonstandard(ipmi_mc_t               *mc,
 int
 ipmi_controls_destroy(ipmi_control_info_t *controls)
 {
-    int j;
+    unsigned int j;
 
     if (controls->destroyed)
 	return EINVAL;
@@ -885,7 +936,7 @@ ipmi_control_set_val(ipmi_control_t     *control,
 		     ipmi_control_op_cb handler,
 		     void               *cb_data)
 {
-    if (control->destroyed)
+    if (!control_ok_to_use(control))
 	return ECANCELED;
       
     CHECK_CONTROL_LOCK(control);
@@ -900,7 +951,7 @@ ipmi_control_get_val(ipmi_control_t      *control,
 		     ipmi_control_val_cb handler,
 		     void                *cb_data)
 {
-    if (control->destroyed)
+    if (!control_ok_to_use(control))
 	return ECANCELED;
       
     CHECK_CONTROL_LOCK(control);
@@ -919,7 +970,7 @@ ipmi_control_set_display_string(ipmi_control_t     *control,
 				ipmi_control_op_cb handler,
 				void               *cb_data)
 {
-    if (control->destroyed)
+    if (!control_ok_to_use(control))
 	return ECANCELED;
       
     CHECK_CONTROL_LOCK(control);
@@ -941,11 +992,11 @@ ipmi_control_get_display_string(ipmi_control_t      *control,
 				ipmi_control_str_cb handler,
 				void                *cb_data)
 {
-    CHECK_CONTROL_LOCK(control);
-
-    if (control->destroyed)
+    if (!control_ok_to_use(control))
 	return ECANCELED;
       
+    CHECK_CONTROL_LOCK(control);
+
     if (!control->cbs.get_display_string)
 	return ENOSYS;
     return control->cbs.get_display_string(control,
@@ -960,7 +1011,7 @@ ipmi_control_identifier_get_val(ipmi_control_t                 *control,
 				ipmi_control_identifier_val_cb handler,
 				void                           *cb_data)
 {
-    if (control->destroyed)
+    if (!control_ok_to_use(control))
 	return ECANCELED;
       
     CHECK_CONTROL_LOCK(control);
@@ -977,7 +1028,7 @@ ipmi_control_identifier_set_val(ipmi_control_t     *control,
 				ipmi_control_op_cb handler,
 				void               *cb_data)
 {
-    if (control->destroyed)
+    if (!control_ok_to_use(control))
 	return ECANCELED;
       
     CHECK_CONTROL_LOCK(control);
@@ -1184,9 +1235,13 @@ control_val_event_call_handler(void *cb_data, void *item1, void *item2)
 		      info->vals,
 		      item2,
 		      info->event);
-    if (handled == IPMI_EVENT_HANDLED) {
-	info->handled = handled;
-	info->event = NULL;
+    if (handled != IPMI_EVENT_NOT_HANDLED) {
+	if (info->handled != IPMI_EVENT_HANDLED)
+	    /* Allow handled to override handled_pass, but not the
+	       other way. */
+	    info->handled = handled;
+	if (handled == IPMI_EVENT_HANDLED)
+	    info->event = NULL;
     }
     return LOCKED_LIST_ITER_CONTINUE;
 }
@@ -1294,7 +1349,7 @@ ipmi_control_get_id(ipmi_control_t *control, char *id, int length)
 
     CHECK_CONTROL_LOCK(control);
 
-    if (control->id_len > length)
+    if ((int) control->id_len > length)
 	clen = length;
     else
 	clen = control->id_len;
@@ -1592,7 +1647,7 @@ typedef struct ipmi_light_s
 
 struct ipmi_light_setting_s
 {
-    unsigned int count;
+    int          count;
     ipmi_light_t *lights;
 };
 
@@ -1743,34 +1798,49 @@ ipmi_light_settings_dup(ipmi_light_setting_t *settings)
     return rv;
 }
 
-void
+int
 ipmi_control_add_light_color_support(ipmi_control_t *control,
+				     int            light_num,
 				     unsigned int   color)
 {
-    control->colors |= (1 << color);
+    if (light_num >= MAX_LIGHTS)
+	return EINVAL;
+    control->colors[light_num] |= (1 << color);
+    return 0;
 }
 
 int
-ipmi_control_light_is_color_supported(ipmi_control_t *control,
-				      unsigned int   color)
+ipmi_control_light_is_color_sup(ipmi_control_t *control,
+				int            light_num,
+				unsigned int   color)
 {
     CHECK_CONTROL_LOCK(control);
 
-    return (control->colors & (1 << color)) != 0;
-}
-
-void
-ipmi_control_light_set_has_local_control(ipmi_control_t *control, int val)
-{
-     control->has_local_control = val;
+    if (light_num >= MAX_LIGHTS)
+	return 0;
+    return (control->colors[light_num] & (1 << color)) != 0;
 }
 
 int
-ipmi_control_light_has_local_control(ipmi_control_t *control)
+ipmi_control_light_set_has_local_control(ipmi_control_t *control,
+					 int            light_num,
+					 int            val)
+{
+    if (light_num >= MAX_LIGHTS)
+	return EINVAL;
+     control->has_local_control[light_num] = val;
+     return 0;
+}
+
+int
+ipmi_control_light_has_loc_ctrl(ipmi_control_t *control,
+				int            light_num)
 {
     CHECK_CONTROL_LOCK(control);
 
-    return control->has_local_control;
+    if (light_num >= MAX_LIGHTS)
+	return 0;
+    return control->has_local_control[light_num];
 }
 
 int
@@ -1909,3 +1979,15 @@ ipmi_control_get_num_light_settings(ipmi_control_t *control,
     return ipmi_control_get_num_light_values(control, light);
 }
 
+int
+ipmi_control_light_is_color_supported(ipmi_control_t *control,
+				      unsigned int   color)
+{
+    return ipmi_control_light_is_color_sup(control, 0, color);
+}
+
+int
+ipmi_control_light_has_local_control(ipmi_control_t *control)
+{
+    return ipmi_control_light_has_loc_ctrl(control, 0);
+}
