@@ -31,6 +31,8 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <config.h>
+
 #ifdef HAVE_OPENIPMI_SMI
 
 #include <sys/ioctl.h>
@@ -43,9 +45,10 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
+#include <stdlib.h>
+#include <ctype.h>
 
 #include <linux/ipmi.h>
-#include <net/af_ipmi.h>
 
 #include <OpenIPMI/ipmi_conn.h>
 #include <OpenIPMI/ipmi_msgbits.h>
@@ -57,6 +60,8 @@
 #include <OpenIPMI/internal/ipmi_int.h>
 #include <OpenIPMI/internal/locked_list.h>
 
+static ipmi_args_t *smi_con_alloc_args(void);
+
 /* We time the SMI messages, but we have a long timer. */
 #define SMI_TIMEOUT 60000
 
@@ -115,7 +120,6 @@ typedef struct smi_data_s
     int                        refcount;
 
     ipmi_con_t                 *ipmi;
-    int                        using_socket;
     int                        fd;
     int                        if_num;
     pending_cmd_t              *pending_cmds;
@@ -126,7 +130,7 @@ typedef struct smi_data_s
     ipmi_lock_t                *smi_lock;
     locked_list_t              *event_handlers;
 
-    unsigned char              slave_addr;
+    unsigned char              slave_addr[MAX_IPMI_USED_CHANNELS];
 
     os_hnd_timer_id_t          *audit_timer;
     audit_timer_info_t         *audit_info;
@@ -384,26 +388,11 @@ remove_cmd_registration(ipmi_con_t    *ipmi,
 }
 
 static int
-open_smi_fd(int if_num, int *using_socket)
+open_smi_fd(int if_num)
 {
-    char                 devname[30];
-    int                  fd;
-    struct sockaddr_ipmi addr;
-    int                  rv;
+    char devname[30];
+    int  fd;
 
-    fd = socket(PF_IPMI, SOCK_DGRAM, 0);
-    if (fd == -1)
-	goto try_dev;
-    addr.sipmi_family = AF_IPMI;
-    addr.if_num = if_num;
-    rv = bind(fd, (struct sockaddr *) &addr, sizeof(addr));
-    if (rv != -1) {
-	*using_socket = 1;
-	goto out;
-    }
-
-    *using_socket = 0;
- try_dev:
     sprintf(devname, "/dev/ipmidev/%d", if_num);
     fd = open(devname, O_RDWR);
     if (fd == -1) {
@@ -415,7 +404,6 @@ open_smi_fd(int if_num, int *using_socket)
 	}
     }
 
- out:
     return fd;
 }
 
@@ -427,12 +415,14 @@ smi_send(smi_data_t        *smi,
 	 const ipmi_msg_t  *msg,
 	 long              msgid)
 {
-    int rv;
-    ipmi_addr_t myaddr;
+    int             rv;
+    ipmi_addr_t     myaddr;
+    struct ipmi_req req;
 
     if (DEBUG_MSG) {
 	char buf1[32], buf2[32];
-	ipmi_log(IPMI_LOG_DEBUG_START, "outgoing msgid=%08lx\n addr =", msgid);
+	ipmi_log(IPMI_LOG_DEBUG_START, "%soutgoing msgid=%08lx\n addr =",
+		 IPMI_CONN_NAME(smi->ipmi), msgid);
 	dump_hex((unsigned char *) addr, addr_len);
         ipmi_log(IPMI_LOG_DEBUG_CONT,
                  "\n msg  = netfn=%s cmd=%s data_len=%d.",
@@ -459,33 +449,12 @@ smi_send(smi_data_t        *smi,
     if (msg->data_len > IPMI_MAX_MSG_LENGTH)
 	return EBADF;
 
-    if (smi->using_socket) {
-	struct sockaddr_ipmi     saddr;
-        char                     smsg_data[sizeof(struct ipmi_sock_msg)
-                                           + IPMI_MAX_MSG_LENGTH];
-        struct ipmi_sock_msg     *smsg = (void *) smsg_data;
-
-	saddr.sipmi_family = AF_IPMI;
-	memcpy(&saddr.ipmi_addr, addr, addr_len);
-
-	smsg->netfn = msg->netfn;
-	smsg->cmd = msg->cmd;
-	smsg->data_len = msg->data_len;
-	smsg->msgid = msgid;
-	memcpy(smsg->data, msg->data, smsg->data_len);
-
-	rv = sendto(fd, smsg, sizeof(*smsg) + msg->data_len, 0,
-		    (struct sockaddr *) &saddr,
-		    addr_len + SOCKADDR_IPMI_OVERHEAD);
-    } else {
-	struct ipmi_req req;
-	req.addr = (unsigned char *) addr;
-	req.addr_len = addr_len;
-	req.msgid = (long) smi;
-	req.msg = *msg;
-	req.msgid = msgid;
-	rv = ioctl(fd, IPMICTL_SEND_COMMAND, &req);
-    }
+    req.addr = (unsigned char *) addr;
+    req.addr_len = addr_len;
+    req.msgid = (long) smi;
+    req.msg = *msg;
+    req.msgid = msgid;
+    rv = ioctl(fd, IPMICTL_SEND_COMMAND, &req);
 
     if (rv == -1)
 	return errno;
@@ -496,13 +465,25 @@ smi_send(smi_data_t        *smi,
 static void
 set_ipmb_in_dev(smi_data_t *smi)
 {
-    unsigned int slave_addr = smi->slave_addr;
-    int          rv;
+    unsigned char                       *slave_addr = smi->slave_addr;
+    unsigned int                        ipmb_addr;
+    struct ipmi_channel_lun_address_set channel_addr;
+    int                                 rv;
+    int                                 i;
 
-    if (smi->using_socket)
-	rv = ioctl(smi->fd, SIOCIPMISETADDR, &slave_addr);
-    else
-	rv = ioctl(smi->fd, IPMICTL_SET_MY_ADDRESS_CMD, &slave_addr);
+    for (i=0; i<MAX_IPMI_USED_CHANNELS; i++) {
+	channel_addr.channel = i;
+	channel_addr.value = slave_addr[i];
+	rv = ioctl(smi->fd, IPMICTL_SET_MY_CHANNEL_ADDRESS_CMD, &channel_addr);
+	if (rv == -1)
+	    goto try_old_version;
+    }
+    return;
+
+ try_old_version:
+    /* We can only set one address. */
+    ipmb_addr = slave_addr[0];
+    rv = ioctl(smi->fd, IPMICTL_SET_MY_ADDRESS_CMD, &ipmb_addr);
     if (rv) {
 	ipmi_log(IPMI_LOG_SEVERE,
 		 "%sipmi_smi.c(set_ipmb_in_dev): "
@@ -513,11 +494,12 @@ set_ipmb_in_dev(smi_data_t *smi)
 
 typedef struct call_ipmb_change_handler_s
 {
-    smi_data_t  *smi;
-    int          err;
-    unsigned int ipmb_addr;
-    int          active;
-    unsigned int hacks;
+    smi_data_t   *smi;
+    int           err;
+    const unsigned char *ipmb_addr;
+    unsigned int  num_ipmb_addr;
+    int           active;
+    unsigned int  hacks;
 } call_ipmb_change_handler_t;
 
 static int
@@ -526,13 +508,15 @@ call_ipmb_change_handler(void *cb_data, void *item1, void *item2)
     call_ipmb_change_handler_t *info = cb_data;
     ipmi_ll_ipmb_addr_cb       handler = item1;
 
-    handler(info->smi->ipmi, info->err, info->ipmb_addr, info->active,
-	    info->hacks, item2);
+    handler(info->smi->ipmi, info->err, info->ipmb_addr, info->num_ipmb_addr,
+	    info->active, info->hacks, item2);
     return LOCKED_LIST_ITER_CONTINUE;
 }
 
 static void
-call_ipmb_change_handlers(smi_data_t *smi, int err, unsigned int ipmb_addr,
+call_ipmb_change_handlers(smi_data_t *smi, int err,
+			  const unsigned char ipmb_addr[],
+			  unsigned int  num_ipmb_addr,
 			  int active, unsigned int hacks)
 {
     call_ipmb_change_handler_t info;
@@ -540,6 +524,7 @@ call_ipmb_change_handlers(smi_data_t *smi, int err, unsigned int ipmb_addr,
     info.smi = smi;
     info.err = err;
     info.ipmb_addr = ipmb_addr;
+    info.num_ipmb_addr = num_ipmb_addr;
     info.active = active;
     info.hacks = hacks;
     locked_list_iterate(smi->ipmb_change_handlers, call_ipmb_change_handler,
@@ -573,22 +558,33 @@ smi_remove_ipmb_addr_handler(ipmi_con_t           *ipmi,
 }
 
 static void
-ipmb_handler(ipmi_con_t   *ipmi,
-	     int          err,
-	     unsigned int ipmb,
-	     int          active,
-	     unsigned int hacks,
-	     void         *cb_data)
+ipmb_handler(ipmi_con_t    *ipmi,
+	     int           err,
+	     const unsigned char ipmb_addr[],
+	     unsigned int  num_ipmb_addr,
+	     int           active,
+	     unsigned int  hacks,
+	     void          *cb_data)
 {
     smi_data_t *smi = (smi_data_t *) ipmi->con_data;
+    int        changed = 0;
+    int        i;
 
     if (err)
 	return;
 
-    if (ipmb != smi->slave_addr) {
-	smi->slave_addr = ipmb;
-	ipmi->ipmb_addr = ipmb;
-	call_ipmb_change_handlers(smi, err, ipmb, active, 0);
+    for (i=0; i<MAX_IPMI_USED_CHANNELS; i++) {
+	if (! ipmb_addr[i])
+	    continue;
+	if (ipmb_addr[i] != smi->slave_addr[i]) {
+	    smi->slave_addr[i] = ipmb_addr[i];
+	    ipmi->ipmb_addr[i] = ipmb_addr[i];
+	    changed = 1;
+	}
+    }
+    if (changed) {
+	call_ipmb_change_handlers(smi, err, ipmb_addr, num_ipmb_addr,
+				  active, 0);
 	set_ipmb_in_dev(smi);
     }
 }
@@ -741,10 +737,7 @@ smi_add_event_handler(ipmi_con_t            *ipmi,
     if (!rv) {
 	if (locked_list_num_entries(smi->event_handlers) == 1) {
 	    int val = 1;
-	    if (smi->using_socket)
-		rv = ioctl(smi->fd, SIOCIPMIGETEVENT, &val);
-	    else
-		rv = ioctl(smi->fd, IPMICTL_SET_GETS_EVENTS_CMD, &val);
+	    rv = ioctl(smi->fd, IPMICTL_SET_GETS_EVENTS_CMD, &val);
 	    if (rv == -1) {
 		locked_list_remove(smi->event_handlers, handler, cb_data);
 		rv = errno;
@@ -768,10 +761,7 @@ smi_remove_event_handler(ipmi_con_t            *ipmi,
 	rv = EINVAL;
     if (locked_list_num_entries(smi->event_handlers) == 0) {
 	int val = 0;
-	if (smi->using_socket)
-	    ioctl(smi->fd, SIOCIPMIGETEVENT, &val);
-	else
-	    ioctl(smi->fd, IPMICTL_SET_GETS_EVENTS_CMD, &val);
+	ioctl(smi->fd, IPMICTL_SET_GETS_EVENTS_CMD, &val);
     }
     ipmi_unlock(smi->smi_lock);
     return rv;
@@ -861,8 +851,8 @@ gen_recv_msg(ipmi_con_t *ipmi, struct ipmi_recv *recv)
 {
     if (DEBUG_MSG) {
 	char buf1[32], buf2[32], buf3[32];
-	ipmi_log(IPMI_LOG_DEBUG_START, "incoming msgid=%08lx\n addr =",
-		 recv->msgid);
+	ipmi_log(IPMI_LOG_DEBUG_START, "%sincoming msgid=%08lx\n addr =",
+		 IPMI_CONN_NAME(ipmi), recv->msgid);
 	dump_hex((unsigned char *) recv->addr, recv->addr_len);
         ipmi_log(IPMI_LOG_DEBUG_CONT,
                  "\n msg  = netfn=%s cmd=%s data_len=%d. cc=%s",
@@ -934,82 +924,6 @@ ipmi_dev_data_handler(int            fd,
     smi_put(ipmi);
 }
 
-static void
-ipmi_sock_data_handler(int            fd,
-		       void           *cb_data,
-		       os_hnd_fd_id_t *id)
-{
-    ipmi_con_t           *ipmi = (ipmi_con_t *) cb_data;
-    struct sockaddr_ipmi addr;
-    socklen_t            addr_len;
-    struct ipmi_sock_msg *smsg;
-    unsigned char        data[MAX_IPMI_DATA_SIZE + sizeof(*smsg)];
-    int                  rv;
-    struct ipmi_recv     recv;
-
-    if (!smi_valid_ipmi(ipmi)) {
-	/* We can have due to a race condition, just return and
-           everything should be fine. */
-	return;
-    }
-
-    addr_len = sizeof(addr);
-    rv = recvfrom(fd, data, sizeof(data), 0,
-		  (struct sockaddr *) &addr, &addr_len);
-    if (rv == -1) {
-	/* FIXME - no handling for EMSGSIZE. */
-	if (errno == EINTR) {
-	    goto out; /* Try again later. */
-	} else {
-	    ipmi_log(IPMI_LOG_SEVERE,
-		     "%sipmi_smi.c(ipmi_sock_data_handler): "
-		     "Error receiving message: %s",
-		     IPMI_CONN_NAME(ipmi), strerror(errno));
-	    goto out;
-	}
-    }
-    if (DEBUG_MSG) {
-	ipmi_log(IPMI_LOG_DEBUG_START, "incoming\n addr(%d.) = ", addr_len);
-	dump_hex((unsigned char *) &addr, MIN(addr_len,sizeof(addr)));
-	if (rv) {
-	    ipmi_log(IPMI_LOG_DEBUG_CONT, "\n data(%d.) =\n  ", rv);
-	    dump_hex(data, rv);
-	}
-	ipmi_log(IPMI_LOG_DEBUG_END, " ");
-    }
-
-    if (rv < sizeof(*smsg)) {
-	ipmi_log(IPMI_LOG_SEVERE,
-		 "%sipmi_smi.c(ipmi_sock_data_handler): "
-		 "Undersized socket message: %d bytes",
-		 IPMI_CONN_NAME(ipmi), rv);
-	goto out;
-    }
-
-    smsg = (struct ipmi_sock_msg *) data;
-    if (rv < (sizeof(*smsg) + smsg->data_len)) {
-	ipmi_log(IPMI_LOG_SEVERE,
-		 "%sipmi_smi.c(ipmi_sock_data_handler): "
-		 "Got subsized msg, size was %d, data_len was %d",
-		 IPMI_CONN_NAME(ipmi), rv, smsg->data_len);
-	goto out;
-    }
-
-    recv.recv_type = smsg->recv_type;
-    recv.addr = (unsigned char *) &addr.ipmi_addr;
-    recv.addr_len = addr_len - SOCKADDR_IPMI_OVERHEAD;
-    recv.msgid = smsg->msgid;
-    recv.msg.netfn = smsg->netfn;
-    recv.msg.cmd = smsg->cmd;
-    recv.msg.data = smsg->data;
-    recv.msg.data_len = smsg->data_len;
-
-    gen_recv_msg(ipmi, &recv);
-
- out:
-    smi_put(ipmi);
-}
-
 static int
 smi_send_command(ipmi_con_t            *ipmi,
 		 const ipmi_addr_t     *addr,
@@ -1052,7 +966,10 @@ smi_send_command(ipmi_con_t            *ipmi,
     {
 	ipmi_ipmb_addr_t *ipmb = (ipmi_ipmb_addr_t *) addr;
 
-	if (ipmb->slave_addr == smi->slave_addr) {
+	if (ipmb->channel >= MAX_IPMI_USED_CHANNELS)
+	    return EINVAL;
+
+	if (ipmb->slave_addr == smi->slave_addr[ipmb->channel]) {
 	    ipmi_system_interface_addr_t *si = (void *) &tmp_addr;
 	    /* Most systems don't handle sending to your own slave
                address, so we have to translate here. */
@@ -1135,10 +1052,7 @@ smi_register_for_command(ipmi_con_t            *ipmi,
 
     reg.netfn = netfn;
     reg.cmd = cmd;
-    if (smi->using_socket)
-	rv = ioctl(smi->fd, SIOCIPMIREGCMD, &reg);
-    else
-	rv = ioctl(smi->fd, IPMICTL_REGISTER_FOR_CMD, &reg);
+    rv = ioctl(smi->fd, IPMICTL_REGISTER_FOR_CMD, &reg);
     if (rv == -1) {
 	remove_cmd_registration(ipmi, netfn, cmd);
 	return errno;
@@ -1161,10 +1075,7 @@ smi_deregister_for_command(ipmi_con_t    *ipmi,
 
     reg.netfn = netfn;
     reg.cmd = cmd;
-    if (smi->using_socket)
-	rv = ioctl(smi->fd, SIOCIPMIUNREGCMD, &reg);
-    else
-	rv = ioctl(smi->fd, IPMICTL_UNREGISTER_FOR_CMD, &reg);
+    rv = ioctl(smi->fd, IPMICTL_UNREGISTER_FOR_CMD, &reg);
     if (rv == -1) {
 	rv = errno;
 	goto out_unlock;
@@ -1211,11 +1122,11 @@ cleanup_con(ipmi_con_t *ipmi)
 
     if (ipmi) {
 	ipmi_con_attr_cleanup(ipmi);
-	ipmi_mem_free(ipmi);
 	if (ipmi->name) {
 	    ipmi_mem_free(ipmi->name);
 	    ipmi->name = NULL;
 	}
+	ipmi_mem_free(ipmi);
     }
 
     if (smi) {
@@ -1305,21 +1216,31 @@ finish_start_con(void *cb_data, os_hnd_timer_id_t *id)
 
     ipmi->os_hnd->free_timer(ipmi->os_hnd, id);
 
-    call_con_change_handlers(smi, 0, 1, 1);
+    call_con_change_handlers(smi, 0, 0, 1);
 }
 
 static void
 smi_set_ipmb_addr(ipmi_con_t    *ipmi,
-		  unsigned char ipmb,
+		  const unsigned char ipmb_addr[],
+		  unsigned int  num_ipmb_addr,
 		  int           active,
 		  unsigned int  hacks)
 {
-    smi_data_t *smi = (smi_data_t *) ipmi->con_data;
+    smi_data_t   *smi = (smi_data_t *) ipmi->con_data;
+    int          changed = 0;
+    unsigned int i;
 
-    if (smi->slave_addr != ipmb) {
-	smi->slave_addr = ipmb;
-	ipmi->ipmb_addr = ipmb;
-	call_ipmb_change_handlers(smi, 0, ipmb, active, 0);
+    for (i=0; i<num_ipmb_addr && i<MAX_IPMI_USED_CHANNELS; i++) {
+	if (! ipmb_addr[i])
+	    continue;
+	if (smi->slave_addr[i] != ipmb_addr[i]) {
+	    smi->slave_addr[i] = ipmb_addr[i];
+	    ipmi->ipmb_addr[i] = ipmb_addr[i];
+	    changed = 1;
+	}
+    }
+    if (changed) {
+	call_ipmb_change_handlers(smi, 0, ipmb_addr, num_ipmb_addr, active, 0);
 	set_ipmb_in_dev(smi);
     }
 }
@@ -1356,24 +1277,30 @@ finish_connection(ipmi_con_t *ipmi, smi_data_t *smi)
 }
 
 static void
-handle_ipmb_addr(ipmi_con_t   *ipmi,
-		 int          err,
-		 unsigned int ipmb_addr,
-		 int          active,
-		 unsigned int hacks,
-		 void         *cb_data)
+handle_ipmb_addr(ipmi_con_t    *ipmi,
+		 int           err,
+		 const unsigned char ipmb_addr[],
+		 unsigned int  num_ipmb_addr,
+		 int           active,
+		 unsigned int  hacks,
+		 void          *cb_data)
 {
-    smi_data_t *smi = (smi_data_t *) ipmi->con_data;
+    smi_data_t   *smi = (smi_data_t *) ipmi->con_data;
+    unsigned int i;
 
     if (err) {
 	call_con_change_handlers(smi, err, 0, 0);
 	return;
     }
 
-    smi->slave_addr = ipmb_addr;
-    ipmi->ipmb_addr = ipmb_addr;
+    for (i=0; i<num_ipmb_addr && i<MAX_IPMI_USED_CHANNELS; i++) {
+	if (! ipmb_addr[i])
+	    continue;
+	smi->slave_addr[i] = ipmb_addr[i];
+	ipmi->ipmb_addr[i] = ipmb_addr[i];
+    }
     finish_connection(ipmi, smi);
-    call_ipmb_change_handlers(smi, err, ipmb_addr, active, 0);
+    call_ipmb_change_handlers(smi, err, ipmb_addr, num_ipmb_addr, active, 0);
     set_ipmb_in_dev(smi);
 }
 
@@ -1482,6 +1409,8 @@ smi_start_con(ipmi_con_t *ipmi)
     return rv;
 }
 
+static ipmi_args_t *get_startup_args(ipmi_con_t *ipmi);
+
 static int
 setup(int          if_num,
       os_handler_t *handlers,
@@ -1491,6 +1420,7 @@ setup(int          if_num,
     ipmi_con_t *ipmi = NULL;
     smi_data_t *smi = NULL;
     int        rv;
+    int        i;
 
     /* Keep things sane. */
     if (if_num >= 100)
@@ -1505,7 +1435,6 @@ setup(int          if_num,
     ipmi->os_hnd = handlers;
     ipmi->con_type = "smi";
     ipmi->priv_level = IPMI_PRIVILEGE_ADMIN; /* Always admin privilege. */
-    ipmi->ipmb_addr = 0x20; /* Assume this until told otherwise */
 
     rv = ipmi_con_attr_init(ipmi);
     if (rv)
@@ -1522,9 +1451,10 @@ setup(int          if_num,
 
     smi->refcount = 1;
     smi->ipmi = ipmi;
-    smi->slave_addr = 0x20; /* Assume this until told otherwise. */
+    for (i=0; i<MAX_IPMI_USED_CHANNELS; i++)
+	smi->slave_addr[i] = 0x20; /* Assume this until told otherwise. */
 
-    smi->fd = open_smi_fd(if_num, &smi->using_socket);
+    smi->fd = open_smi_fd(if_num);
     if (smi->fd == -1) {
 	rv = errno;
 	goto out_err;
@@ -1578,22 +1508,14 @@ setup(int          if_num,
     ipmi->close_connection = smi_close_connection;
     ipmi->close_connection_done = smi_close_connection_done;
     ipmi->handle_async_event = handle_async_event;
+    ipmi->get_startup_args = get_startup_args;
 
-    if (smi->using_socket) {
-	rv = handlers->add_fd_to_wait_for(ipmi->os_hnd,
-					  smi->fd,
-					  ipmi_sock_data_handler, 
-					  ipmi,
-					  NULL,
-					  &(smi->fd_wait_id));
-    } else {
-	rv = handlers->add_fd_to_wait_for(ipmi->os_hnd,
-					  smi->fd,
-					  ipmi_dev_data_handler,
-					  ipmi,
-					  NULL,
-					  &(smi->fd_wait_id));
-    }
+    rv = handlers->add_fd_to_wait_for(ipmi->os_hnd,
+				      smi->fd,
+				      ipmi_dev_data_handler,
+				      ipmi,
+				      NULL,
+				      &(smi->fd_wait_id));
     if (rv) {
 	goto out_err;
     }
@@ -1634,6 +1556,198 @@ ipmi_smi_setup_con(int          if_num,
     return err;
 }
 
+typedef struct smi_args_s
+{
+    int ifnum;
+} smi_args_t;
+
+static ipmi_args_t *
+get_startup_args(ipmi_con_t *ipmi)
+{
+    ipmi_args_t *args;
+    smi_args_t  *sargs;
+    smi_data_t  *smi;
+
+    args = smi_con_alloc_args();
+    if (! args)
+	return NULL;
+    sargs = _ipmi_args_get_extra_data(args);
+    smi = (smi_data_t *) ipmi->con_data;
+    sargs->ifnum = smi->if_num;
+    return args;
+}
+
+static int
+smi_connect_args(ipmi_args_t  *args,
+		 os_handler_t *handler,
+		 void         *user_data,
+		 ipmi_con_t   **new_con)
+{
+    smi_args_t *sargs = _ipmi_args_get_extra_data(args);
+
+    return ipmi_smi_setup_con(sargs->ifnum, handler, user_data, new_con);
+}
+
+static const char *
+smi_args_get_type(ipmi_args_t  *args)
+{
+    return "smi";
+}
+
+static int
+smi_args_get_val(ipmi_args_t  *args,
+		 unsigned int argnum,
+		 const char   **name,
+		 const char   **type,
+		 const char   **help,
+		 char         **value,
+		 const char   ***range)
+{
+    smi_args_t *sargs = _ipmi_args_get_extra_data(args);
+    char       dummy[1];
+    char       *sval;
+
+    if (argnum > 0)
+	return E2BIG;
+
+    if (name)
+	*name = "Interface_Number";
+    if (type)
+	*type = "str";
+    if (help)
+	*help = "*The interface number to open.  For instance, /dev/ipmi0"
+	    " would be 0.  This is an integer value.";
+    if (*value) {
+	int len;
+	len = snprintf(dummy, 1, "%d", sargs->ifnum);
+	sval = ipmi_mem_alloc(len+1);
+	if (! sval)
+	    return ENOMEM;
+	len = snprintf(sval, len+1, "%d", sargs->ifnum);
+	*value = sval;
+    }
+    return 0;
+}
+
+static int
+smi_args_set_val(ipmi_args_t  *args,
+		 unsigned int argnum,
+		 const char   *name,
+		 const char   *value)
+{
+    smi_args_t   *sargs = _ipmi_args_get_extra_data(args);
+    const char   *should_be_end;
+    char         *end;
+    unsigned int val;
+
+    if (name) {
+	if (strcmp(name, "Interface_Number") != 0)
+	    return EINVAL;
+    } else if (argnum > 0) {
+	return E2BIG;
+    }
+
+    if (!value)
+	return EINVAL;
+
+    should_be_end = value + strlen(value) - 1;
+    while ((should_be_end >= value) && isspace(*should_be_end))
+	should_be_end--;
+    should_be_end++;
+    if (should_be_end <= value)
+	return EINVAL;
+
+    val = strtoul(value, &end, 0);
+    if (end != should_be_end)
+	return EINVAL;
+    sargs->ifnum = val;
+    return 0;
+}
+
+static ipmi_args_t *
+smi_args_copy(ipmi_args_t *args)
+{
+    ipmi_args_t *nargs;
+    smi_args_t  *sargs = _ipmi_args_get_extra_data(args);
+    smi_args_t  *nsargs;
+
+    nargs = smi_con_alloc_args();
+    if (!nargs)
+	return NULL;
+    nsargs = _ipmi_args_get_extra_data(nargs);
+    *nsargs = *sargs;
+    return nargs;
+}
+
+static int
+smi_args_validate(ipmi_args_t *args, int *argnum)
+{
+    return 1; /* Can't be invalid */
+}
+
+#define CHECK_ARG \
+    do { \
+        if (*curr_arg >= arg_count) { \
+	    rv = EINVAL; \
+	    goto out_err; \
+        } \
+    } while(0)
+
+static int
+smi_parse_args(int         *curr_arg,
+	       int         arg_count,
+	       char        * const *args,
+	       ipmi_args_t **iargs)
+{
+    int         rv;
+    ipmi_args_t *p = NULL;
+    smi_args_t  *sargs;
+
+    CHECK_ARG;
+
+    p = smi_con_alloc_args();
+    if (!p)
+	return ENOMEM;
+
+    sargs = _ipmi_args_get_extra_data(p);
+    sargs->ifnum = atoi(args[*curr_arg]);
+    *iargs = p;
+    (*curr_arg)++;
+    return 0;
+
+ out_err:
+    if (p)
+	ipmi_free_args(p);
+    return rv;
+}
+
+static void
+smi_args_free_val(ipmi_args_t *args, char *value)
+{
+    ipmi_mem_free(value);
+}
+
+static const char *
+smi_parse_help(void)
+{
+    return
+	"\n"
+	" smi <num>\n"
+	"where the <num> is the IPMI device number to connect to.";
+}
+
+static ipmi_args_t *
+smi_con_alloc_args(void)
+{
+    return _ipmi_args_alloc(NULL, smi_connect_args,
+			    smi_args_get_val, smi_args_set_val,
+			    smi_args_copy, smi_args_validate,
+			    smi_args_free_val, smi_args_get_type,
+			    sizeof(smi_args_t));
+}
+
+static ipmi_con_setup_t *smi_setup;
+
 int
 _ipmi_smi_init(os_handler_t *os_hnd)
 {
@@ -1642,12 +1756,31 @@ _ipmi_smi_init(os_handler_t *os_hnd)
     rv = ipmi_create_global_lock(&smi_list_lock);
     if (rv)
 	return rv;
+
+    smi_setup = _ipmi_alloc_con_setup(smi_parse_args, smi_parse_help,
+				      smi_con_alloc_args);
+    if (! smi_setup) {
+	ipmi_destroy_lock(smi_list_lock);
+	return ENOMEM;
+    }
+    rv = _ipmi_register_con_type("smi", smi_setup);
+    if (rv) {
+	_ipmi_free_con_setup(smi_setup);
+	smi_setup = NULL;
+	ipmi_destroy_lock(smi_list_lock);
+	return rv;
+    }
+
     return 0;
 }
 
 void
 _ipmi_smi_shutdown(void)
 {
+    _ipmi_unregister_con_type("smi", smi_setup);
+    _ipmi_free_con_setup(smi_setup);
+    smi_setup = NULL;
+
     if (smi_list_lock) {
 	ipmi_destroy_lock(smi_list_lock);
 	smi_list_lock = NULL;
