@@ -52,34 +52,36 @@ atca_ipmb_handler(ipmi_con_t *ipmi, ipmi_msgi_t *rspi)
     ipmi_msg_t           *msg = &rspi->msg;
     ipmi_ll_ipmb_addr_cb handler = rspi->data1;
     void                 *cb_data = rspi->data2;
-    unsigned char        ipmb = 0;
+    unsigned char        ipmb[MAX_IPMI_USED_CHANNELS];
     int                  err = 0;
     atca_conn_info_t     *info;
 
     if (!ipmi) {
 	if (handler)
-	    handler(ipmi, ECANCELED, ipmb, 1, 0, cb_data);
+	    handler(ipmi, ECANCELED, ipmb, 1, 1, 0, cb_data);
 	return IPMI_MSG_ITEM_NOT_USED;
     }
 
     info = ipmi->oem_data;
 
+    memset(ipmb, 0, sizeof(*ipmb));
+
     if (msg->data[0] != 0) 
 	err = IPMI_IPMI_ERR_VAL(msg->data[0]);
     else if (msg->data_len < 4)
 	err = EINVAL;
     else if ((msg->data[7] == 3) && (!info->dont_use_floating_addr))
-	ipmb = 0x20; /* This is a Dedicated ShMC and we are not doing
-			dual-ShMC addressing. */
+	ipmb[0] = 0x20; /* This is a Dedicated ShMC and we are not doing
+			   dual-ShMC addressing. */
     else
-	ipmb = msg->data[3];
+	ipmb[0] = msg->data[3];
 
     /* Note that there is no "inactive" connection with ATCA. */
     if (!err)
-	ipmi->set_ipmb_addr(ipmi, ipmb, 1, info->hacks);
+	ipmi->set_ipmb_addr(ipmi, ipmb, 1, 1, info->hacks);
 
     if (handler)
-	handler(ipmi, err, ipmb, 1, info->hacks, cb_data);
+	handler(ipmi, err, ipmb, 1, 1, info->hacks, cb_data);
     return IPMI_MSG_ITEM_NOT_USED;
 }
 
@@ -133,7 +135,8 @@ atca_oem_finish_check(ipmi_con_t *ipmi, ipmi_msgi_t *rspi)
     ipmi_conn_oem_check_done done = rspi->data1;
     void                     *cb_data = rspi->data2;
 
-    if (ipmi && (msg->data_len >= 8) && (msg->data[0] == 0)) {
+    if (ipmi && !ipmi->oem_data && (msg->data_len >= 8) && (msg->data[0] == 0))
+    {
 	atca_conn_info_t *info;
 
 	info = ipmi_mem_alloc(sizeof(*info));
@@ -200,6 +203,9 @@ handle_intel_atca(ipmi_con_t *conn, void *cb_data)
 {
     atca_conn_info_t *info = conn->oem_data;
 
+    if (!info)
+	return 0;
+
     /* This means that we don't advertise 0x20 as our address on the
        CMM, we use the real address. */
 #if 0
@@ -223,6 +229,16 @@ ipmi_oem_atca_conn_init(void)
     if (rv)
 	return rv;
 
+    rv = ipmi_register_oem_conn_handler(0x000157, 0x080b,
+					handle_intel_atca, NULL);
+    if (rv)
+	return rv;
+
+    rv = ipmi_register_oem_conn_handler(0x000157, 0x080c,
+					handle_intel_atca, NULL);
+    if (rv)
+	return rv;
+
     return 0;
 }
 
@@ -231,4 +247,6 @@ ipmi_oem_atca_conn_shutdown(void)
 {
     ipmi_deregister_conn_oem_check(atca_oem_check, NULL);
     ipmi_deregister_oem_conn_handler(0x000157, 0x0841);
+    ipmi_deregister_oem_conn_handler(0x000157, 0x080c);
+    ipmi_deregister_oem_conn_handler(0x000157, 0x080b);
 }
