@@ -31,6 +31,8 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <config.h>
+
 #include <string.h>
 
 #include <OpenIPMI/ipmi_conn.h>
@@ -98,8 +100,16 @@ check_rakp_rsp(ipmi_con_t   *ipmi,
 	       unsigned int min_length,
 	       int          addr_num)
 {
-    if (!ipmi) {
+    if (!ipmi)
 	return ECANCELED;
+
+    if (msg->data_len == 1) {
+	/* This is kind of a cheap hack, this can happen when there is
+	   a timeout. */
+	ipmi_log(IPMI_LOG_ERR_INFO,
+		 "rakp.c(%s): IPMI error: %d",
+		 caller, msg->data[0]);
+	return IPMI_IPMI_ERR_VAL(msg->data[0]);
     }
 
     if (msg->data_len < 2) {
@@ -109,10 +119,9 @@ check_rakp_rsp(ipmi_con_t   *ipmi,
 	return EINVAL;
     }
 
-    if (msg->data[1]) {
+    if (msg->data[1])
 	/* Got an RMCP+ error. */
 	return IPMI_RMCPP_ERR_VAL(msg->data[1]);
-    }
 
     if (msg->data_len < min_length) {
 	ipmi_log(IPMI_LOG_ERR_INFO,
@@ -527,7 +536,7 @@ rakp_hmac_s3(rakp_info_t   *info,
     ipmi_set_uint32(idata+16, ipmi_rmcpp_auth_get_my_session_id(info->ainfo));
     idata[20] = ipmi_rmcpp_auth_get_role(info->ainfo);
     if (info->hacks & IPMI_CONN_HACK_RAKP3_WRONG_ROLEM)
-	/* For the RAKP4 message, the Intel BMC only uses the bottom 4
+	/* For the RAKP3 message, the Intel BMC only uses the bottom 4
 	   nibbles. */
 	idata[20] &= 0xf;
     idata[21] = ipmi_rmcpp_auth_get_username_len(info->ainfo);
