@@ -38,6 +38,7 @@
 #include <OpenIPMI/ipmi_auth.h>
 #include <OpenIPMI/ipmiif.h>
 #include <OpenIPMI/ipmi_err.h>
+#include <OpenIPMI/ipmi_mc.h>
 #include <string.h>
 #include <OpenIPMI/internal/ipmi_malloc.h>
 
@@ -593,6 +594,7 @@ static char *sensor_states[256][16] =
 	"undetermined system hardware failure",
 	"entry added to auxiliary log",
 	"PEF action",
+	"Timestamp clock sync"
     },
     { /* 0x13 */
 	"front panel NMI/diag interrupt",
@@ -627,7 +629,10 @@ static char *sensor_states[256][16] =
 	"initiated by hard reset",
 	"initiated by warm reset",
 	"user requested PXE boot",
-	"automatic boot to diagnostic"
+	"automatic boot to diagnostic",
+	"OS initiated hard reset",
+	"OS initiated warm reset",
+	"System restart",
     },
     { /* 0x1e */
 	"no bootable media",
@@ -647,7 +652,10 @@ static char *sensor_states[256][16] =
     },
     { /* 0x20 */
 	"stop during OS load/initialization",
-	"run time stop"
+	"run time critical stop",
+	"OS Graceful Stop",
+	"Soft shutdown initiated by PEF",
+	"Agent not responding, graceful shutdown did not occur"
     },
     { /* 0x21 */
 	"fault status asserted",
@@ -709,6 +717,8 @@ static char *sensor_states[256][16] =
 	"controller access degraded or unavailable",
 	"management controller off-line",
 	"management controller unavailable",
+	"sensor failure",
+	"FRU failure"
     },
     { /* 0x29 */
 	"battery low",
@@ -1192,95 +1202,129 @@ ipmi_get_cc_string(unsigned int cc,
     return buffer;
 }
 
+int
+ipmi_get_cc_string_len(unsigned int cc)
+{
+    char *cc_fs;
+    char dummy[1];
+
+    if ( cc == 0x00 )
+	cc_fs = ipmi_ccodes[0];
+    else if ( (cc > 0xbf) && (cc < 0xd6) )
+	cc_fs = ipmi_ccodes[cc - 0xc0 + 1];
+    else if ( cc == 0xff )
+	cc_fs = ipmi_ccodes[23];
+    else
+	cc_fs = "Unknown:%02x";
+
+    return snprintf(dummy, 1, cc_fs, cc);
+}
+
 
 static const char *sol_error_codes[] =
 {
-	"SoLCharTransUnavail",
-	"SoLDeactivated",
-	"SoLNotAvailable",
-	"SoLDisconnected",
-	"SoLUnconfirmOp",
-	"SoLFlushed",
-	"SoLUnknown"
+    "SoLCharTransUnavail",
+    "SoLDeactivated",
+    "SoLNotAvailable",
+    "SoLDisconnected",
+    "SoLUnconfirmOp",
+    "SoLFlushed",
+    "SoLUnknown"
 };
 
 static const char *rmcpp_error_codes[] =
 {
-	"InsufResourForSess",
-	"InvalSessID",
-	"InvalPayloadType",
-	"InvalAuthAlg",
-	"InvalIntegAlg",
-	"NoMatchAuthPayld",
-	"NoMatchIntegPayld",
-	"InactSessID",
-	"InvalidRole",
-	"UnauthRoleOrPriv",
-	"InsufResourForRole",
-	"InvalNameLength",
-	"UnauthName",
-	"UnauthGUID",
-	"InvalIntegChkVal",
-	"InvalConfidAlg",
-	"NoCipherSuiteMatch",
-	"IllegalParam",
-	"RMCPPUnknown"
+    "InsufResourForSess",
+    "InvalSessID",
+    "InvalPayloadType",
+    "InvalAuthAlg",
+    "InvalIntegAlg",
+    "NoMatchAuthPayld",
+    "NoMatchIntegPayld",
+    "InactSessID",
+    "InvalidRole",
+    "UnauthRoleOrPriv",
+    "InsufResourForRole",
+    "InvalNameLength",
+    "UnauthName",
+    "UnauthGUID",
+    "InvalIntegChkVal",
+    "InvalConfidAlg",
+    "NoCipherSuiteMatch",
+    "IllegalParam",
+    "RMCPPUnknown"
 };
 
 char *
 ipmi_get_error_string(unsigned int err,
-			char *buffer,
-			unsigned int buf_len)
+		      char         *buffer,
+		      unsigned int buf_len)
 {
-	char *temp_buffer;
-	char *err_type;
-
-	if (err == 0)
-	{
-		strncpy(buffer, "Success (No error)", buf_len);
-		return buffer;
-	}
-
-	temp_buffer = ipmi_mem_alloc(buf_len);
-	if (!temp_buffer)
-		temp_buffer = buffer;
-
-	if (IPMI_IS_OS_ERR(err))
-	{
-		strncpy(temp_buffer, strerror(IPMI_GET_OS_ERR(err)), buf_len);
-		err_type = "OS";
-	} else if (IPMI_IS_IPMI_ERR(err))
-	{
-		ipmi_get_cc_string(IPMI_GET_IPMI_ERR(err), temp_buffer, buf_len);
-		err_type = "IPMI";
-	} else if (IPMI_IS_RMCPP_ERR(err))
-	{
-		int rmcpp_err = IPMI_GET_RMCPP_ERR(err);
-		if ((rmcpp_err <= 0) && (rmcpp_err > 0x12))
-			rmcpp_err = 0x13;
-		snprintf(temp_buffer, buf_len, "%s (0x%02x)", rmcpp_error_codes[rmcpp_err - 1], IPMI_GET_RMCPP_ERR(err));
-		err_type = "RMCP+";
-	} else if (IPMI_IS_SOL_ERR(err))
-	{
-		int sol_err = IPMI_GET_SOL_ERR(err);
-		if ((sol_err < 1) || (sol_err > 7))
-			sol_err = 7;
-		snprintf(temp_buffer, buf_len, sol_error_codes[sol_err - 1]);
-		err_type = "SoL";
-	}
-	else
-	{
-		strncpy(temp_buffer, "Unknown", buf_len);
-		err_type = "Unknown";
-	}
-
-	if (temp_buffer != buffer)
-	{
-		snprintf(buffer, buf_len, "%s: %s", err_type, temp_buffer);
-		ipmi_mem_free(temp_buffer);
-	}
+    unsigned int len;
+    char         *err_type;
 
+    if (err == 0) {
+	strncpy(buffer, "Success (No error)", buf_len);
 	return buffer;
+    }
+
+    if (IPMI_IS_OS_ERR(err)) {
+	snprintf(buffer+4, buf_len-4, "%s", strerror(IPMI_GET_OS_ERR(err)));
+	err_type = "OS: ";
+    } else if (IPMI_IS_IPMI_ERR(err)) {
+	ipmi_get_cc_string(IPMI_GET_IPMI_ERR(err), buffer+6, buf_len-6);
+	err_type = "IPMI: ";
+    } else if (IPMI_IS_RMCPP_ERR(err)) {
+	int rmcpp_err = IPMI_GET_RMCPP_ERR(err);
+	if ((rmcpp_err <= 0) && (rmcpp_err > 0x12))
+	    rmcpp_err = 0x13;
+	snprintf(buffer+7, buf_len-7, "%s (0x%02x)",
+		 rmcpp_error_codes[rmcpp_err - 1],
+		 IPMI_GET_RMCPP_ERR(err));
+	err_type = "RMCP+: ";
+    } else if (IPMI_IS_SOL_ERR(err)) {
+	int sol_err = IPMI_GET_SOL_ERR(err);
+	if ((sol_err < 1) || (sol_err > 7))
+	    sol_err = 7;
+	strncpy(buffer+5, sol_error_codes[sol_err - 1], buf_len-5);
+	err_type = "SoL: ";
+    } else {
+	strncpy(buffer+9, "Unknown", buf_len-9);
+	err_type = "Unknown: ";
+    }
+
+    len = strlen(err_type);
+    if (len > buf_len-1) {
+	len = buf_len-1;
+	buffer[buf_len-1] = '\0';
+    }
+    memcpy(buffer, err_type, len);
+
+    return buffer;
+}
+
+int ipmi_get_error_string_len(unsigned int err)
+{
+    if (err == 0)
+	return(strlen("Success (No error)"));
+
+    if (IPMI_IS_OS_ERR(err))
+	return strlen(strerror(IPMI_GET_OS_ERR(err))) + 5;
+    else if (IPMI_IS_IPMI_ERR(err)) {
+	return ipmi_get_cc_string_len(IPMI_GET_IPMI_ERR(err)) + 7;
+    } else if (IPMI_IS_RMCPP_ERR(err)) {
+	int rmcpp_err = IPMI_GET_RMCPP_ERR(err);
+	if ((rmcpp_err <= 0) && (rmcpp_err > 0x12))
+	    rmcpp_err = 0x13;
+	return strlen(rmcpp_error_codes[rmcpp_err - 1]) + 15;
+    } else if (IPMI_IS_SOL_ERR(err)) {
+	int sol_err = IPMI_GET_SOL_ERR(err);
+	if ((sol_err < 1) || (sol_err > 7))
+	    sol_err = 7;
+	return strlen(sol_error_codes[sol_err - 1]) + 6;
+    } else {
+	return strlen("Unknown") + 10;
+    }
 }
 
 /* Get a string name fo the hot swap state. */
@@ -1305,6 +1349,7 @@ static char *domain_types[] =
     "unknown",
     "MXP",
     "ATCA",
+    "ATCA_BLADE",
 };
 #define NUM_DOMAIN_TYPES (sizeof(domain_types)/sizeof(char *))
 
@@ -1357,3 +1402,95 @@ ipmi_privilege_string(int privilege)
 	return "invalid";
     }
 }
+
+const char *
+ipmi_channel_medium_string(int val)
+{
+    switch (val) {
+    case IPMI_CHANNEL_MEDIUM_IPMB:
+	return "IPMB";
+    case IPMI_CHANNEL_MEDIUM_ICMB_V10:
+	return "ICMB_V10";
+    case IPMI_CHANNEL_MEDIUM_ICMB_V09:
+	return "ICMB_V09";
+    case IPMI_CHANNEL_MEDIUM_8023_LAN:
+	return "8023_LAN";
+    case IPMI_CHANNEL_MEDIUM_RS232:
+	return "RS232";
+    case IPMI_CHANNEL_MEDIUM_OTHER_LAN:
+	return "OTHER_LAN";
+    case IPMI_CHANNEL_MEDIUM_PCI_SMBUS:
+	return "PCI_SMBUS";
+    case IPMI_CHANNEL_MEDIUM_SMBUS_v1:
+	return "SMBUS_v1";
+    case IPMI_CHANNEL_MEDIUM_SMBUS_v2:
+	return "SMBUS_v2";
+    case IPMI_CHANNEL_MEDIUM_USB_v1:
+	return "USB_v1";
+    case IPMI_CHANNEL_MEDIUM_USB_v2:
+	return "USB_v2";
+    case IPMI_CHANNEL_MEDIUM_SYS_INTF:
+	return "SYS_INTF";
+    default:
+	return "invalid";
+    }
+}
+
+const char *
+ipmi_channel_protocol_string(int val)
+{
+    switch (val) {
+    case IPMI_CHANNEL_PROTOCOL_IPMB:
+	return "IPMB";
+    case IPMI_CHANNEL_PROTOCOL_ICMB:
+	return "ICMB";
+    case IPMI_CHANNEL_PROTOCOL_SMBus:
+	return "SMBus";
+    case IPMI_CHANNEL_PROTOCOL_KCS:
+	return "KCS";
+    case IPMI_CHANNEL_PROTOCOL_SMIC:
+	return "SMIC";
+    case IPMI_CHANNEL_PROTOCOL_BT_v10:
+	return "BT_v10";
+    case IPMI_CHANNEL_PROTOCOL_BT_v15:
+	return "BT_v15";
+    case IPMI_CHANNEL_PROTOCOL_TMODE:
+	return "TMODE";
+    default:
+	return "invalid";
+    }
+}
+
+const char *
+ipmi_channel_session_support_string(int val)
+{
+    switch (val) {
+    case IPMI_CHANNEL_SESSION_LESS:
+	return "session-less";
+    case IPMI_CHANNEL_SINGLE_SESSION:
+	return "single-session";
+    case IPMI_CHANNEL_MULTI_SESSION:
+	return "multi-session";
+    case IPMI_CHANNEL_SESSION_BASED:
+	return "session-based";
+    default:
+	return "invalid";
+    }
+}
+
+const char *
+ipmi_channel_access_mode_string(int val)
+{
+    switch (val) {
+    case IPMI_CHANNEL_ACCESS_MODE_DISABLED:
+	return "DISABLED";
+    case IPMI_CHANNEL_ACCESS_MODE_PRE_BOOT:
+	return "PRE_BOOT";
+    case IPMI_CHANNEL_ACCESS_MODE_ALWAYS:
+	return "ALWAYS";
+    case IPMI_CHANNEL_ACCESS_MODE_SHARED:
+	return "SHARED";
+    default:
+	return "invalid";
+    }
+}
