@@ -45,7 +45,6 @@
 #include <netdb.h>
 #include <stdlib.h>
 #include <string.h>
-#include <popt.h> /* Option parsing made easy */
 
 #include <OpenIPMI/ipmi_conn.h>
 #include <OpenIPMI/ipmi_lan.h>
@@ -61,40 +60,10 @@
 void ipmi_oem_force_conn_init(void);
 int ipmi_oem_motorola_mxp_init(void);
 
-static char* sOp		= NULL;
 static int   interactive        = 1;
 static int   interactive_done   = 0;
 
-struct poptOption poptOpts[]=
-{
-    {
-	"command",
-	'k',
-	POPT_ARG_STRING,
-	&sOp,
-	'k',
-	"Command string to be send",
-	""
-    },
-    {
-	"version",
-	'v',
-	POPT_ARG_NONE,
-	NULL,
-	'v',
-	"Display version info about the program",
-	NULL
-    },
-    POPT_AUTOHELP
-    {
-	NULL,
-	0,
-	0,
-	NULL,
-	0		 
-    }	
-};
-
+char *progname;
 selector_t *sel;
 os_handler_t *os_hnd;
 static ipmi_con_t *con;
@@ -136,15 +105,27 @@ leave(int ret)
     exit(ret);
 }
 
-void printInfo( )
+void printInfo(void)
 {
     printf("ipmicmd\n");
     printf("This little utility is an ipmi command tool ;-)\n");
     printf("It can be used to send commands to an IPMI interface\n");
-    printf("It uses popt for command line parsing, type -? for usage info.\n");
+    printf("type -? for usage info.\n");
     printf("Enjoy!\n");
 }
 
+void con_usage(const char *name, const char *help, void *cb_data)
+{
+    printf("\n%s%s", name, help);
+}
+
+void usage(void)
+{
+    printf("%s [-k <command>] [-v] <con_parms>\n", progname);
+    printf("Where <con_parms> is one of:");
+    ipmi_parse_args_iter_help(con_usage, NULL);
+}
+
 char *
 get_addr_type(int type)
 {
@@ -272,7 +253,7 @@ event_handler(ipmi_con_t        *ipmi,
     unsigned int        type = ipmi_event_get_type(event);
     unsigned int        data_len = ipmi_event_get_data_len(event);
     const unsigned char *data = ipmi_event_get_data_ptr(event);
-    int                 i;
+    unsigned int        i;
 
     printf("Got event:\n");
     printf("  %4.4x (%2.2x):", record_id, type);
@@ -382,12 +363,12 @@ process_input_line(char *buf)
     char               *strtok_data;
     char               *endptr;
     char               *v = strtok_r(buf, " \t\r\n,.\"", &strtok_data);
-    int                pos = 0;
+    unsigned int       pos = 0;
     int                start;
     ipmi_addr_t        addr;
     unsigned int       addr_len;
     ipmi_msg_t         msg;
-    char               outbuf[MAX_IPMI_DATA_SIZE];
+    unsigned char      outbuf[MAX_IPMI_DATA_SIZE];
     int                rv = 0;
     short              channel;
     unsigned char      seq = 0;
@@ -404,11 +385,11 @@ process_input_line(char *buf)
 	printf("  help - This help\n");
 	printf("  0f <lun> <netfn> <cmd> <data.....> - send a command\n");
 	printf("      to the local BMC\n");
-	printf("  <channel> [ipmb] <dest addr> <lun> <netfn> <cmd> <data...> -\n");
-	printf("      send an IPMB command on the channel.\n");
-	printf("  <channel> lan <handle> <remote swid> <local swid> <lun> <netfn> <cmd> <data...> -\n");
-	printf("      send a command on a LAN channel.\n");
-	printf("  <channel> 00 <dest addr> <lun> <netfn> <cmd> <data...> -\n");
+	printf("  [ipmb] <channel> <dest addr> <lun> <netfn> [seq] <cmd> <data...> -\n");
+	printf("      send an IPMB command on the channel.  seq is used if this is a response\n");
+	printf("  lan <channel> <handle> <remote swid> <local swid> <lun> <netfn> [seq] <cmd> <data...> -\n");
+	printf("      send a command on a LAN channel.  seq is used if this is a response\n");
+	printf("  [ipmb] <channel> 00 <dest addr> <lun> <netfn> <cmd> <data...> -\n");
 	printf("      broadcast a command on the channel.\n");
 	printf("  test_lat <count> <command> - Send the command and wait for\n"
 	       "      the response <count> times and measure the average\n"
@@ -639,7 +620,7 @@ user_input_ready(int fd, void *data)
     }
 }
 
-char buf[256];
+char *cmdstr;
 
 static void
 con_changed_handler(ipmi_con_t   *ipmi,
@@ -655,7 +636,7 @@ con_changed_handler(ipmi_con_t   *ipmi,
 	}
 	if (!interactive_done) {
 	    interactive_done = 1;
-	    process_input_line(buf);
+	    process_input_line(cmdstr);
 	}
     } else {
 	if (err)
@@ -673,51 +654,14 @@ main(int argc, char *argv[])
 {
     int         rv;
     int         pos;
-    int         o;
-    char        *bufline = NULL;
     int         curr_arg;
     ipmi_args_t *args;
-    const char  **oargv = (const char **) argv;
+    int         i;
 
+    progname = argv[0];
 
-    poptContext poptCtx = poptGetContext("ipmicmd", argc, oargv, poptOpts, 0);
-
-    while (( o = poptGetNextOpt(poptCtx)) >= 0)
-    {   
-	switch( o )
-	{
-	    case 'k':
-		strcpy( buf, poptGetOptArg(poptCtx) );
-		bufline = buf;
-		interactive = 0;
-		break;
-
-	    case 'v':
-		printInfo();
-		exit(0);
-		break;
-
-	    default:
-		poptPrintUsage(poptCtx, stderr, 0);
-		exit(1);
-		break;
-	}
-    }
-
-    argv = (char **) poptGetArgs(poptCtx);
-
-    if (!argv) {
-	fprintf(stderr, "Not enough arguments\n");
-	exit(1);
-    }
-
-    for (argc=0; argv[argc]!= NULL; argc++)
-	;
-
-    if (argc < 1) {
-	fprintf(stderr, "Not enough arguments\n");
-	exit(1);
-    }
+    /* Have to initalize this first so the usage help will work, since
+       it needs OpenIPMI initialized. */
 
     /* OS handler allocated first. */
     os_hnd = ipmi_posix_get_os_handler();
@@ -739,9 +683,45 @@ main(int argc, char *argv[])
 	exit(1);
     }
 
-    curr_arg = 0;
+    for (i=1; i<argc; i++) {
+	if (argv[i][0] != '-')
+	    break;
+	if (strcmp(argv[i], "--") == 0) {
+	    i++;
+	    break;
+	} else if ((strcmp(argv[i], "-k") == 0)
+		   || (strcmp(argv[i], "--command") == 0))
+	{
+	    i++;
+	    if (i >= argc) {
+		usage();
+		exit(1);
+	    }
+	    cmdstr = argv[i];
+	    interactive = 0;
+	} else if ((strcmp(argv[i], "-v") == 0)
+		   || (strcmp(argv[i], "--version") == 0))
+	{
+	    printInfo();
+	    exit(0);
+	} else {
+	    usage();
+	    exit(1);
+	}
+    }
 
-    rv = ipmi_parse_args(&curr_arg, argc, argv, &args);
+    if (i >= argc) {
+	fprintf(stderr, "Not enough arguments\n");
+	exit(1);
+    }
+
+    curr_arg = i;
+
+    if (strcmp(argv[0], "ipmicmd") == 0)
+	/* Backwards compatible interface */
+	rv = ipmi_parse_args(&curr_arg, argc, argv, &args);
+    else
+	rv = ipmi_parse_args2(&curr_arg, argc, argv, &args);
     if (rv) {
 	fprintf(stderr, "Error parsing command arguments, argument %d: %s\n",
 		curr_arg, strerror(rv));
